"use strict";
/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
Object.defineProperty(exports, "__esModule", { value: true });
var core_1 = require("@angular/core");
/**
 * Conditionally apply event listeners to an element
 *
 * Usage:
 * <button [ev-condition]="isActive()"
 *      ev-events="click"
 *      (ev-fire)="handleClick($event)">...</button>
 *
 * Notes:
 * The [ev-events] property can be either a string or an array of strings to handle multiple events
 */
var ConditionEventDirective = (function () {
    function ConditionEventDirective(elm, renderer) {
        this.elm = elm;
        this.renderer = renderer;
        this.fire = new core_1.EventEmitter();
        this.listeners = [];
    }
    /**
     * @param {?} changes
     * @return {?}
     */
    ConditionEventDirective.prototype.ngOnChanges = /**
     * @param {?} changes
     * @return {?}
     */
    function (changes) {
        var _this = this;
        this.unregisterAll();
        if (!this.condition || !this.events)
            return;
        var /** @type {?} */ events = (typeof (this.events) === 'string')
            ? [this.events]
            : this.events;
        this.listeners = events.map(function (x) { return _this.addListener(x); });
    };
    /**
     * @return {?}
     */
    ConditionEventDirective.prototype.ngOnDestroy = /**
     * @return {?}
     */
    function () {
        this.unregisterAll();
    };
    /**
     * @return {?}
     */
    ConditionEventDirective.prototype.unregisterAll = /**
     * @return {?}
     */
    function () {
        this.listeners.forEach(function (x) { return x(); });
        this.listeners = [];
    };
    /**
     * @param {?} eventName
     * @return {?}
     */
    ConditionEventDirective.prototype.addListener = /**
     * @param {?} eventName
     * @return {?}
     */
    function (eventName) {
        var _this = this;
        var /** @type {?} */ colon = eventName.indexOf(':');
        var /** @type {?} */ handler = function ($event) { return _this.fire.next($event); };
        if (colon === -1) {
            return this.renderer.listen(this.elm.nativeElement, eventName, handler);
        }
        else {
            var /** @type {?} */ scope = eventName.slice(0, colon);
            var /** @type {?} */ realEventName = eventName.slice(colon + 1);
            return this.renderer.listen(scope, realEventName, handler);
        }
    };
    ConditionEventDirective.decorators = [
        { type: core_1.Directive, args: [{
                    selector: '[ev-condition]'
                },] },
    ];
    /** @nocollapse */
    ConditionEventDirective.ctorParameters = function () { return [
        { type: core_1.ElementRef, },
        { type: core_1.Renderer2, },
    ]; };
    ConditionEventDirective.propDecorators = {
        "condition": [{ type: core_1.Input, args: ['ev-condition',] },],
        "events": [{ type: core_1.Input, args: ['ev-events',] },],
        "fire": [{ type: core_1.Output, args: ['ev-fire',] },],
    };
    return ConditionEventDirective;
}());
exports.ConditionEventDirective = ConditionEventDirective;
function ConditionEventDirective_tsickle_Closure_declarations() {
    /** @type {!Array<{type: !Function, args: (undefined|!Array<?>)}>} */
    ConditionEventDirective.decorators;
    /**
     * @nocollapse
     * @type {function(): !Array<(null|{type: ?, decorators: (undefined|!Array<{type: !Function, args: (undefined|!Array<?>)}>)})>}
     */
    ConditionEventDirective.ctorParameters;
    /** @type {!Object<string,!Array<{type: !Function, args: (undefined|!Array<?>)}>>} */
    ConditionEventDirective.propDecorators;
    /** @type {?} */
    ConditionEventDirective.prototype.condition;
    /** @type {?} */
    ConditionEventDirective.prototype.events;
    /** @type {?} */
    ConditionEventDirective.prototype.fire;
    /** @type {?} */
    ConditionEventDirective.prototype.listeners;
    /** @type {?} */
    ConditionEventDirective.prototype.elm;
    /** @type {?} */
    ConditionEventDirective.prototype.renderer;
}
