import { Directive, ElementRef, EventEmitter, Inject, Injectable, InjectionToken, Input, NgModule, NgZone, Output, Renderer2 } from '@angular/core';
import { EVENT_MANAGER_PLUGINS } from '@angular/platform-browser';
import { DOCUMENT } from '@angular/common';
import { Subject as Subject$1 } from 'rxjs/Subject';
import { auditTime } from 'rxjs/operators';

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
/**
 * @abstract
 */
class MyEventManagerPlugin {
    /**
     * @param {?} _doc
     */
    constructor(_doc) {
        this._doc = _doc;
    }
    /**
     * @param {?} element
     * @param {?} eventName
     * @param {?} handler
     * @return {?}
     */
    addGlobalEventListener(element, eventName, handler) {
        let /** @type {?} */ target;
        if (element === 'document')
            target = this._doc;
        else if (element === 'window' && typeof (window) !== undefined)
            target = window;
        if (!target) {
            throw new Error(`Unsupported event target ${target} for event ${eventName}`);
        }
        return this.addEventListener(target, eventName, handler);
    }
    ;
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
/**
 * Listen to events that are fired outside of the current element and its children
 *
 * Usage:
 * <div (outside.click)="close()"></div>
 *
 */
class OutsideEventPlugin extends MyEventManagerPlugin {
    /**
     * @param {?} doc
     */
    constructor(doc) {
        super(doc);
    }
    /**
     * @param {?} eventName
     * @return {?}
     */
    supports(eventName) {
        return eventName.indexOf('outside.') === 0;
    }
    /**
     * @param {?} element
     * @param {?} eventName
     * @param {?} handler
     * @return {?}
     */
    addEventListener(element, eventName, handler) {
        const /** @type {?} */ realEventName = eventName.slice(8);
        return this.manager.addGlobalEventListener('document', realEventName, (e) => {
            if (element !== e.target && !element.contains(/** @type {?} */ (e.target))) {
                handler(e);
            }
        });
    }
}
OutsideEventPlugin.decorators = [
    { type: Injectable },
];
/** @nocollapse */
OutsideEventPlugin.ctorParameters = () => [
    { type: undefined, decorators: [{ type: Inject, args: [DOCUMENT,] },] },
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
class OutsideEventModule {
}
OutsideEventModule.decorators = [
    { type: NgModule, args: [{
                providers: [
                    {
                        provide: EVENT_MANAGER_PLUGINS,
                        useClass: OutsideEventPlugin,
                        multi: true
                    }
                ]
            },] },
];
/** @nocollapse */
OutsideEventModule.ctorParameters = () => [];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
/**
 * Listen to events without triggering change detection
 *
 * Usage:
 * <button (undetected.click)="handleClick()"></button>
 *
 * ...
 *
 * export class ExampleComponent implements OnInit {
 *
 *  constructor(private zone: NgZone) {}
 *
 *  handleClick() {
 *      if(someCondition) {
 *          this.zone.run(() => {
 *              ...
 *          });
 *      }
 *  }
 * }
 *
 */
class UndetectedEventPlugin extends MyEventManagerPlugin {
    /**
     * @param {?} doc
     */
    constructor(doc) {
        super(doc);
    }
    /**
     * @param {?} eventName
     * @return {?}
     */
    supports(eventName) {
        return eventName.indexOf('undetected.') === 0;
    }
    /**
     * @param {?} element
     * @param {?} eventName
     * @param {?} handler
     * @return {?}
     */
    addEventListener(element, eventName, handler) {
        const /** @type {?} */ realEventName = eventName.slice(11);
        return this.manager.getZone().runOutsideAngular(() => this.manager.addEventListener(element, realEventName, handler));
    }
}
UndetectedEventPlugin.decorators = [
    { type: Injectable },
];
/** @nocollapse */
UndetectedEventPlugin.ctorParameters = () => [
    { type: undefined, decorators: [{ type: Inject, args: [DOCUMENT,] },] },
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
class UndetectedEventModule {
}
UndetectedEventModule.decorators = [
    { type: NgModule, args: [{
                providers: [
                    {
                        provide: EVENT_MANAGER_PLUGINS,
                        useClass: UndetectedEventPlugin,
                        multi: true
                    }
                ]
            },] },
];
/** @nocollapse */
UndetectedEventModule.ctorParameters = () => [];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
/**
 * Register multiple event listeners
 *
 * Usage:
 * <button (multi.focus,select)="foo($event)"></button>
 *
 */
class MultiEventPlugin extends MyEventManagerPlugin {
    /**
     * @param {?} doc
     */
    constructor(doc) {
        super(doc);
    }
    /**
     * @param {?} eventName
     * @return {?}
     */
    supports(eventName) {
        return eventName.indexOf('multi.') === 0;
    }
    /**
     * @param {?} element
     * @param {?} eventName
     * @param {?} handler
     * @return {?}
     */
    addEventListener(element, eventName, handler) {
        const /** @type {?} */ eventNames = eventName.slice(6).split(',');
        const /** @type {?} */ eventListeners = eventNames.map(x => this.manager.addEventListener(element, x, handler));
        return () => {
            eventListeners.forEach(x => x());
        };
    }
}
MultiEventPlugin.decorators = [
    { type: Injectable },
];
/** @nocollapse */
MultiEventPlugin.ctorParameters = () => [
    { type: undefined, decorators: [{ type: Inject, args: [DOCUMENT,] },] },
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
class MultiEventModule {
}
MultiEventModule.decorators = [
    { type: NgModule, args: [{
                providers: [
                    {
                        provide: EVENT_MANAGER_PLUGINS,
                        useClass: MultiEventPlugin,
                        multi: true
                    }
                ]
            },] },
];
/** @nocollapse */
MultiEventModule.ctorParameters = () => [];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
/**
 * Control event listeners with an observable operator without triggering change detection
 * every time the event fires
 *
 * Usage:
 * <button (observe.throttleTime-500.click)="foo()"></button>
 *
 * The method used must be present on the Observable object
 * e.g. through an explicit import:
 * import 'rxjs/add/operator/throttleTime';
 *
 *
 * To get finer-grained control and the possibility to add multiple observable operators
 * use the ObserveEventDirective.
 */
class ObserveEventPlugin extends MyEventManagerPlugin {
    /**
     * @param {?} doc
     */
    constructor(doc) {
        super(doc);
    }
    /**
     * @param {?} eventName
     * @return {?}
     */
    supports(eventName) {
        return eventName.indexOf('observe.') === 0;
    }
    /**
     * @param {?} element
     * @param {?} eventName
     * @param {?} handler
     * @return {?}
     */
    addEventListener(element, eventName, handler) {
        return this.manager.getZone().runOutsideAngular(() => {
            const /** @type {?} */ eventRest = eventName.slice(8).split('.');
            const /** @type {?} */ fnParts = eventRest[0].split('-');
            const /** @type {?} */ fnName = fnParts[0];
            const /** @type {?} */ fnArg = (fnParts.length > 1) ? +fnParts[1] : undefined;
            const /** @type {?} */ delegateEvent = eventRest.slice(1).join('.');
            const /** @type {?} */ subject = new Subject$1();
            let /** @type {?} */ observable = subject.asObservable();
            if (observable[fnName]) {
                observable = (fnArg !== undefined)
                    ? observable[fnName](fnArg)
                    : observable[fnName]();
            }
            else {
                console.error('The function "' + fnName + '" does not exist on Observable!');
                return () => { };
            }
            observable.subscribe(x => this.manager.getZone().run(() => handler(x)));
            const /** @type {?} */ listener = this.manager.addEventListener(element, delegateEvent, ($event) => {
                subject.next($event);
            });
            return () => {
                listener();
                subject.complete();
            };
        });
    }
}
ObserveEventPlugin.decorators = [
    { type: Injectable },
];
/** @nocollapse */
ObserveEventPlugin.ctorParameters = () => [
    { type: undefined, decorators: [{ type: Inject, args: [DOCUMENT,] },] },
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
class ObserveEventModule {
}
ObserveEventModule.decorators = [
    { type: NgModule, args: [{
                providers: [
                    {
                        provide: EVENT_MANAGER_PLUGINS,
                        useClass: ObserveEventPlugin,
                        multi: true
                    }
                ]
            },] },
];
/** @nocollapse */
ObserveEventModule.ctorParameters = () => [];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
/**
 * Pass events to a given observable subject without triggering change detection
 *
 * Usage:
 * <button [ev-observe]="subject" ev-events="click"></button>
 *
 * ...
 *
 * export class ExampleComponent implements OnInit {
 *  public subject = new Subject();
 *
 *  constructor(private zone: NgZone) {}
 *
 *  ngOnInit() {
 *      this.subject
 *          .throttleTime(300)
 *          .filter(someCondition)
 *          ...
 *          .subscribe($event => this.zone.run(() => this.handleEvent($event)));
 *  }
 * }
 *
 * Notes:
 * The [ev-events] property can be either a string or an array of strings to handle multiple events
 *
 */
class ObserveEventDirective {
    /**
     * @param {?} elm
     * @param {?} renderer
     * @param {?} zone
     */
    constructor(elm, renderer, zone) {
        this.elm = elm;
        this.renderer = renderer;
        this.zone = zone;
        this.listeners = [];
    }
    /**
     * @param {?} e
     * @return {?}
     */
    set events(e) {
        this.unregisterAll();
        const /** @type {?} */ events = (typeof (e) === 'string')
            ? [e]
            : e;
        this.zone.runOutsideAngular(() => {
            this.listeners = events.map(x => this.addListener(x));
        });
    }
    /**
     * @return {?}
     */
    ngOnDestroy() {
        this.unregisterAll();
    }
    /**
     * @return {?}
     */
    unregisterAll() {
        this.listeners.forEach(x => x());
        this.listeners = [];
    }
    /**
     * @param {?} eventName
     * @return {?}
     */
    addListener(eventName) {
        const /** @type {?} */ colon = eventName.indexOf(':');
        const /** @type {?} */ handler = ($event) => this.fireEvent($event);
        if (colon === -1) {
            return this.renderer.listen(this.elm.nativeElement, eventName, handler);
        }
        else {
            const /** @type {?} */ scope = eventName.slice(0, colon);
            const /** @type {?} */ realEventName = eventName.slice(colon + 1);
            return this.renderer.listen(scope, realEventName, handler);
        }
    }
    /**
     * @param {?} $event
     * @return {?}
     */
    fireEvent($event) {
        if (!this.observe)
            return;
        this.observe.next($event);
    }
}
ObserveEventDirective.decorators = [
    { type: Directive, args: [{
                selector: '[ev-observe]'
            },] },
];
/** @nocollapse */
ObserveEventDirective.ctorParameters = () => [
    { type: ElementRef, },
    { type: Renderer2, },
    { type: NgZone, },
];
ObserveEventDirective.propDecorators = {
    "observe": [{ type: Input, args: ['ev-observe',] },],
    "events": [{ type: Input, args: ['ev-events',] },],
};

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
class ObserveEventDirectiveModule {
}
ObserveEventDirectiveModule.decorators = [
    { type: NgModule, args: [{
                declarations: [
                    ObserveEventDirective
                ],
                exports: [
                    ObserveEventDirective
                ]
            },] },
];
/** @nocollapse */
ObserveEventDirectiveModule.ctorParameters = () => [];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
/**
 * Quick-firing 'up' and 'down' events that work cross-browser for mouse and touch events
 *
 * Usage:
 * <button (down)="activate()" (up)="deactivate()"></button>
 *
 */
class TouchEventPlugin extends MyEventManagerPlugin {
    /**
     * @param {?} doc
     */
    constructor(doc) {
        super(doc);
    }
    /**
     * @param {?} eventName
     * @return {?}
     */
    supports(eventName) {
        return eventName === 'down' || eventName === 'up';
    }
    /**
     * @param {?} element
     * @param {?} eventName
     * @param {?} handler
     * @return {?}
     */
    addEventListener(element, eventName, handler) {
        const /** @type {?} */ eventNames = (eventName === 'down')
            ? ['mousedown', 'touchstart', 'pointerdown']
            : ['mouseup', 'touchend', 'pointerup'];
        const /** @type {?} */ eventListeners = eventNames.map(x => this.manager.addEventListener(element, x, (e) => {
            // prevent default so only one of the event listeners is fired
            e.preventDefault();
            handler(e);
        }));
        return () => {
            eventListeners.forEach(x => x());
        };
    }
}
TouchEventPlugin.decorators = [
    { type: Injectable },
];
/** @nocollapse */
TouchEventPlugin.ctorParameters = () => [
    { type: undefined, decorators: [{ type: Inject, args: [DOCUMENT,] },] },
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
class TouchEventModule {
}
TouchEventModule.decorators = [
    { type: NgModule, args: [{
                providers: [
                    {
                        provide: EVENT_MANAGER_PLUGINS,
                        useClass: TouchEventPlugin,
                        multi: true
                    }
                ]
            },] },
];
/** @nocollapse */
TouchEventModule.ctorParameters = () => [];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
/**
 * Automatically unregister an event listener after the event fired once
 *
 * Usage:
 * <button (once.click)="foo()">...</button>
 *
 */
class OnceEventPlugin extends MyEventManagerPlugin {
    /**
     * @param {?} doc
     */
    constructor(doc) {
        super(doc);
    }
    /**
     * @param {?} eventName
     * @return {?}
     */
    supports(eventName) {
        return eventName.indexOf('once.') === 0;
    }
    /**
     * @param {?} element
     * @param {?} eventName
     * @param {?} handler
     * @return {?}
     */
    addEventListener(element, eventName, handler) {
        const /** @type {?} */ realEventName = eventName.slice(5);
        let /** @type {?} */ active = true;
        const /** @type {?} */ eventListener = this.manager.addEventListener(element, realEventName, ($event) => {
            eventListener();
            active = false;
            handler($event);
        });
        return () => {
            if (active)
                eventListener();
        };
    }
}
OnceEventPlugin.decorators = [
    { type: Injectable },
];
/** @nocollapse */
OnceEventPlugin.ctorParameters = () => [
    { type: undefined, decorators: [{ type: Inject, args: [DOCUMENT,] },] },
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
class OnceEventModule {
}
OnceEventModule.decorators = [
    { type: NgModule, args: [{
                providers: [
                    {
                        provide: EVENT_MANAGER_PLUGINS,
                        useClass: OnceEventPlugin,
                        multi: true
                    }
                ]
            },] },
];
/** @nocollapse */
OnceEventModule.ctorParameters = () => [];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
/**
 * Conditionally apply event listeners to an element
 *
 * Usage:
 * <button [ev-condition]="isActive()"
 *      ev-events="click"
 *      (ev-fire)="handleClick($event)">...</button>
 *
 * Notes:
 * The [ev-events] property can be either a string or an array of strings to handle multiple events
 */
class ConditionEventDirective {
    /**
     * @param {?} elm
     * @param {?} renderer
     */
    constructor(elm, renderer) {
        this.elm = elm;
        this.renderer = renderer;
        this.fire = new EventEmitter();
        this.listeners = [];
    }
    /**
     * @param {?} changes
     * @return {?}
     */
    ngOnChanges(changes) {
        this.unregisterAll();
        if (!this.condition || !this.events)
            return;
        const /** @type {?} */ events = (typeof (this.events) === 'string')
            ? [this.events]
            : this.events;
        this.listeners = events.map(x => this.addListener(x));
    }
    /**
     * @return {?}
     */
    ngOnDestroy() {
        this.unregisterAll();
    }
    /**
     * @return {?}
     */
    unregisterAll() {
        this.listeners.forEach(x => x());
        this.listeners = [];
    }
    /**
     * @param {?} eventName
     * @return {?}
     */
    addListener(eventName) {
        const /** @type {?} */ colon = eventName.indexOf(':');
        const /** @type {?} */ handler = ($event) => this.fire.next($event);
        if (colon === -1) {
            return this.renderer.listen(this.elm.nativeElement, eventName, handler);
        }
        else {
            const /** @type {?} */ scope = eventName.slice(0, colon);
            const /** @type {?} */ realEventName = eventName.slice(colon + 1);
            return this.renderer.listen(scope, realEventName, handler);
        }
    }
}
ConditionEventDirective.decorators = [
    { type: Directive, args: [{
                selector: '[ev-condition]'
            },] },
];
/** @nocollapse */
ConditionEventDirective.ctorParameters = () => [
    { type: ElementRef, },
    { type: Renderer2, },
];
ConditionEventDirective.propDecorators = {
    "condition": [{ type: Input, args: ['ev-condition',] },],
    "events": [{ type: Input, args: ['ev-events',] },],
    "fire": [{ type: Output, args: ['ev-fire',] },],
};

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
class ConditionEventDirectiveModule {
}
ConditionEventDirectiveModule.decorators = [
    { type: NgModule, args: [{
                declarations: [
                    ConditionEventDirective
                ],
                exports: [
                    ConditionEventDirective
                ]
            },] },
];
/** @nocollapse */
ConditionEventDirectiveModule.ctorParameters = () => [];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
const SCROLL_EVENT_TIME = new InjectionToken('ScrollEventTime');
/**
 * Detects when an element is scrolled into or out of the viewport
 *
 * Usage:
 * <div (scroll-in)="activate()" (scroll-out)="deactivate()">...</div>
 *
 * The matching handler of the initial status is called upon attaching
 * to notify the element of its current status. $event is true for the
 * initial call, false otherwise
 *
 */
class ScrollEventPlugin extends MyEventManagerPlugin {
    /**
     * @param {?} doc
     * @param {?} time
     */
    constructor(doc, time) {
        super(doc);
        this.listeners = [];
        this.globalListener = undefined;
        this.subject = new Subject$1();
        this.subject
            .pipe(auditTime(time))
            .subscribe(() => {
            this.listeners.forEach(x => x());
        });
    }
    /**
     * @param {?} eventName
     * @return {?}
     */
    supports(eventName) {
        return eventName === 'scroll-in' || eventName === 'scroll-out';
    }
    /**
     * @param {?} element
     * @param {?} eventName
     * @param {?} handler
     * @return {?}
     */
    addEventListener(element, eventName, handler) {
        const /** @type {?} */ isScrollIn = (eventName === 'scroll-in');
        let /** @type {?} */ status = undefined;
        setTimeout(() => {
            status = this.getStatus(element);
            if ((isScrollIn && status) || (!isScrollIn && !status)) {
                handler(true);
            }
        }, 0);
        const /** @type {?} */ listener = () => {
            const /** @type {?} */ newStatus = this.getStatus(element);
            if (status === newStatus)
                return;
            if ((isScrollIn && newStatus) || (!isScrollIn && !newStatus)) {
                this.manager.getZone().run(() => handler(false));
            }
            status = newStatus;
        };
        this.listeners.push(listener);
        this.updateGlobalListener();
        return () => {
            const /** @type {?} */ index = this.listeners.indexOf(listener);
            if (index === -1)
                return;
            this.listeners.splice(index, 1);
            this.updateGlobalListener();
        };
    }
    /**
     * @return {?}
     */
    updateGlobalListener() {
        if (this.listeners.length && this.globalListener === undefined) {
            this.manager.getZone().runOutsideAngular(() => {
                const /** @type {?} */ handler = () => {
                    this.subject.next();
                };
                const /** @type {?} */ listeners = ['scroll', 'resize', 'orientationchange']
                    .map(x => this.manager.addGlobalEventListener('window', x, handler));
                this.globalListener = () => {
                    listeners.forEach(x => x());
                };
            });
        }
        else if (!this.listeners.length && this.globalListener !== undefined) {
            this.globalListener();
            this.globalListener = undefined;
        }
    }
    /**
     * @param {?} element
     * @return {?}
     */
    getStatus(element) {
        const /** @type {?} */ rect = element.getBoundingClientRect();
        return (rect.bottom >= 0 && rect.top <= window.innerHeight);
    }
}
ScrollEventPlugin.decorators = [
    { type: Injectable },
];
/** @nocollapse */
ScrollEventPlugin.ctorParameters = () => [
    { type: undefined, decorators: [{ type: Inject, args: [DOCUMENT,] },] },
    { type: undefined, decorators: [{ type: Inject, args: [SCROLL_EVENT_TIME,] },] },
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
class ScrollEventModule {
}
ScrollEventModule.decorators = [
    { type: NgModule, args: [{
                providers: [
                    {
                        provide: EVENT_MANAGER_PLUGINS,
                        useClass: ScrollEventPlugin,
                        multi: true
                    },
                    {
                        provide: SCROLL_EVENT_TIME,
                        useValue: 200
                    }
                ]
            },] },
];
/** @nocollapse */
ScrollEventModule.ctorParameters = () => [];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
class Ng2EventsModule$1 {
}
Ng2EventsModule$1.decorators = [
    { type: NgModule, args: [{
                imports: [
                    OutsideEventModule,
                    TouchEventModule,
                    MultiEventModule,
                    UndetectedEventModule,
                    ObserveEventModule,
                    ObserveEventDirectiveModule,
                    OnceEventModule,
                    ConditionEventDirectiveModule,
                    ScrollEventModule
                ],
                exports: [
                    OutsideEventModule,
                    TouchEventModule,
                    MultiEventModule,
                    UndetectedEventModule,
                    ObserveEventModule,
                    ObserveEventDirectiveModule,
                    OnceEventModule,
                    ConditionEventDirectiveModule,
                    ScrollEventModule
                ]
            },] },
];
/** @nocollapse */
Ng2EventsModule$1.ctorParameters = () => [];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */

export { Ng2EventsModule$1 as Ng2EventsModule, OutsideEventModule, UndetectedEventModule, MultiEventModule, ObserveEventModule, ObserveEventDirectiveModule, TouchEventModule, OnceEventModule, ConditionEventDirectiveModule, ScrollEventModule, SCROLL_EVENT_TIME };
export default Ng2EventsModule$1;
