var __extends = (this && this.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
import { Directive, ElementRef, EventEmitter, Inject, Injectable, InjectionToken, Input, NgModule, NgZone, Output, Renderer2 } from '@angular/core';
import { EVENT_MANAGER_PLUGINS } from '@angular/platform-browser';
import { DOCUMENT } from '@angular/common';
import { Subject as Subject$1 } from 'rxjs/Subject';
import { auditTime } from 'rxjs/operators';
/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
/**
 * @abstract
 */
var MyEventManagerPlugin = (function () {
    /**
     * @param {?} _doc
     */
    function MyEventManagerPlugin(_doc) {
        this._doc = _doc;
    }
    /**
     * @param {?} element
     * @param {?} eventName
     * @param {?} handler
     * @return {?}
     */
    MyEventManagerPlugin.prototype.addGlobalEventListener = function (element, eventName, handler) {
        var /** @type {?} */ target;
        if (element === 'document')
            target = this._doc;
        else if (element === 'window' && typeof (window) !== undefined)
            target = window;
        if (!target) {
            throw new Error("Unsupported event target " + target + " for event " + eventName);
        }
        return this.addEventListener(target, eventName, handler);
    };
    ;
    return MyEventManagerPlugin;
}());
/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
/**
 * Listen to events that are fired outside of the current element and its children
 *
 * Usage:
 * <div (outside.click)="close()"></div>
 *
 */
var OutsideEventPlugin = (function (_super) {
    __extends(OutsideEventPlugin, _super);
    /**
     * @param {?} doc
     */
    function OutsideEventPlugin(doc) {
        return _super.call(this, doc) || this;
    }
    /**
     * @param {?} eventName
     * @return {?}
     */
    OutsideEventPlugin.prototype.supports = function (eventName) {
        return eventName.indexOf('outside.') === 0;
    };
    /**
     * @param {?} element
     * @param {?} eventName
     * @param {?} handler
     * @return {?}
     */
    OutsideEventPlugin.prototype.addEventListener = function (element, eventName, handler) {
        var /** @type {?} */ realEventName = eventName.slice(8);
        return this.manager.addGlobalEventListener('document', realEventName, function (e) {
            if (element !== e.target && !element.contains(/** @type {?} */ (e.target))) {
                handler(e);
            }
        });
    };
    return OutsideEventPlugin;
}(MyEventManagerPlugin));
OutsideEventPlugin.decorators = [
    { type: Injectable },
];
/** @nocollapse */
OutsideEventPlugin.ctorParameters = function () { return [
    { type: undefined, decorators: [{ type: Inject, args: [DOCUMENT,] },] },
]; };
/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
var OutsideEventModule = (function () {
    function OutsideEventModule() {
    }
    return OutsideEventModule;
}());
OutsideEventModule.decorators = [
    { type: NgModule, args: [{
                providers: [
                    {
                        provide: EVENT_MANAGER_PLUGINS,
                        useClass: OutsideEventPlugin,
                        multi: true
                    }
                ]
            },] },
];
/** @nocollapse */
OutsideEventModule.ctorParameters = function () { return []; };
/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
/**
 * Listen to events without triggering change detection
 *
 * Usage:
 * <button (undetected.click)="handleClick()"></button>
 *
 * ...
 *
 * export class ExampleComponent implements OnInit {
 *
 *  constructor(private zone: NgZone) {}
 *
 *  handleClick() {
 *      if(someCondition) {
 *          this.zone.run(() => {
 *              ...
 *          });
 *      }
 *  }
 * }
 *
 */
var UndetectedEventPlugin = (function (_super) {
    __extends(UndetectedEventPlugin, _super);
    /**
     * @param {?} doc
     */
    function UndetectedEventPlugin(doc) {
        return _super.call(this, doc) || this;
    }
    /**
     * @param {?} eventName
     * @return {?}
     */
    UndetectedEventPlugin.prototype.supports = function (eventName) {
        return eventName.indexOf('undetected.') === 0;
    };
    /**
     * @param {?} element
     * @param {?} eventName
     * @param {?} handler
     * @return {?}
     */
    UndetectedEventPlugin.prototype.addEventListener = function (element, eventName, handler) {
        var _this = this;
        var /** @type {?} */ realEventName = eventName.slice(11);
        return this.manager.getZone().runOutsideAngular(function () { return _this.manager.addEventListener(element, realEventName, handler); });
    };
    return UndetectedEventPlugin;
}(MyEventManagerPlugin));
UndetectedEventPlugin.decorators = [
    { type: Injectable },
];
/** @nocollapse */
UndetectedEventPlugin.ctorParameters = function () { return [
    { type: undefined, decorators: [{ type: Inject, args: [DOCUMENT,] },] },
]; };
/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
var UndetectedEventModule = (function () {
    function UndetectedEventModule() {
    }
    return UndetectedEventModule;
}());
UndetectedEventModule.decorators = [
    { type: NgModule, args: [{
                providers: [
                    {
                        provide: EVENT_MANAGER_PLUGINS,
                        useClass: UndetectedEventPlugin,
                        multi: true
                    }
                ]
            },] },
];
/** @nocollapse */
UndetectedEventModule.ctorParameters = function () { return []; };
/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
/**
 * Register multiple event listeners
 *
 * Usage:
 * <button (multi.focus,select)="foo($event)"></button>
 *
 */
var MultiEventPlugin = (function (_super) {
    __extends(MultiEventPlugin, _super);
    /**
     * @param {?} doc
     */
    function MultiEventPlugin(doc) {
        return _super.call(this, doc) || this;
    }
    /**
     * @param {?} eventName
     * @return {?}
     */
    MultiEventPlugin.prototype.supports = function (eventName) {
        return eventName.indexOf('multi.') === 0;
    };
    /**
     * @param {?} element
     * @param {?} eventName
     * @param {?} handler
     * @return {?}
     */
    MultiEventPlugin.prototype.addEventListener = function (element, eventName, handler) {
        var _this = this;
        var /** @type {?} */ eventNames = eventName.slice(6).split(',');
        var /** @type {?} */ eventListeners = eventNames.map(function (x) { return _this.manager.addEventListener(element, x, handler); });
        return function () {
            eventListeners.forEach(function (x) { return x(); });
        };
    };
    return MultiEventPlugin;
}(MyEventManagerPlugin));
MultiEventPlugin.decorators = [
    { type: Injectable },
];
/** @nocollapse */
MultiEventPlugin.ctorParameters = function () { return [
    { type: undefined, decorators: [{ type: Inject, args: [DOCUMENT,] },] },
]; };
/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
var MultiEventModule = (function () {
    function MultiEventModule() {
    }
    return MultiEventModule;
}());
MultiEventModule.decorators = [
    { type: NgModule, args: [{
                providers: [
                    {
                        provide: EVENT_MANAGER_PLUGINS,
                        useClass: MultiEventPlugin,
                        multi: true
                    }
                ]
            },] },
];
/** @nocollapse */
MultiEventModule.ctorParameters = function () { return []; };
/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
/**
 * Control event listeners with an observable operator without triggering change detection
 * every time the event fires
 *
 * Usage:
 * <button (observe.throttleTime-500.click)="foo()"></button>
 *
 * The method used must be present on the Observable object
 * e.g. through an explicit import:
 * import 'rxjs/add/operator/throttleTime';
 *
 *
 * To get finer-grained control and the possibility to add multiple observable operators
 * use the ObserveEventDirective.
 */
var ObserveEventPlugin = (function (_super) {
    __extends(ObserveEventPlugin, _super);
    /**
     * @param {?} doc
     */
    function ObserveEventPlugin(doc) {
        return _super.call(this, doc) || this;
    }
    /**
     * @param {?} eventName
     * @return {?}
     */
    ObserveEventPlugin.prototype.supports = function (eventName) {
        return eventName.indexOf('observe.') === 0;
    };
    /**
     * @param {?} element
     * @param {?} eventName
     * @param {?} handler
     * @return {?}
     */
    ObserveEventPlugin.prototype.addEventListener = function (element, eventName, handler) {
        var _this = this;
        return this.manager.getZone().runOutsideAngular(function () {
            var /** @type {?} */ eventRest = eventName.slice(8).split('.');
            var /** @type {?} */ fnParts = eventRest[0].split('-');
            var /** @type {?} */ fnName = fnParts[0];
            var /** @type {?} */ fnArg = (fnParts.length > 1) ? +fnParts[1] : undefined;
            var /** @type {?} */ delegateEvent = eventRest.slice(1).join('.');
            var /** @type {?} */ subject = new Subject$1();
            var /** @type {?} */ observable = subject.asObservable();
            if (observable[fnName]) {
                observable = (fnArg !== undefined)
                    ? observable[fnName](fnArg)
                    : observable[fnName]();
            }
            else {
                console.error('The function "' + fnName + '" does not exist on Observable!');
                return function () { };
            }
            observable.subscribe(function (x) { return _this.manager.getZone().run(function () { return handler(x); }); });
            var /** @type {?} */ listener = _this.manager.addEventListener(element, delegateEvent, function ($event) {
                subject.next($event);
            });
            return function () {
                listener();
                subject.complete();
            };
        });
    };
    return ObserveEventPlugin;
}(MyEventManagerPlugin));
ObserveEventPlugin.decorators = [
    { type: Injectable },
];
/** @nocollapse */
ObserveEventPlugin.ctorParameters = function () { return [
    { type: undefined, decorators: [{ type: Inject, args: [DOCUMENT,] },] },
]; };
/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
var ObserveEventModule = (function () {
    function ObserveEventModule() {
    }
    return ObserveEventModule;
}());
ObserveEventModule.decorators = [
    { type: NgModule, args: [{
                providers: [
                    {
                        provide: EVENT_MANAGER_PLUGINS,
                        useClass: ObserveEventPlugin,
                        multi: true
                    }
                ]
            },] },
];
/** @nocollapse */
ObserveEventModule.ctorParameters = function () { return []; };
/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
/**
 * Pass events to a given observable subject without triggering change detection
 *
 * Usage:
 * <button [ev-observe]="subject" ev-events="click"></button>
 *
 * ...
 *
 * export class ExampleComponent implements OnInit {
 *  public subject = new Subject();
 *
 *  constructor(private zone: NgZone) {}
 *
 *  ngOnInit() {
 *      this.subject
 *          .throttleTime(300)
 *          .filter(someCondition)
 *          ...
 *          .subscribe($event => this.zone.run(() => this.handleEvent($event)));
 *  }
 * }
 *
 * Notes:
 * The [ev-events] property can be either a string or an array of strings to handle multiple events
 *
 */
var ObserveEventDirective = (function () {
    /**
     * @param {?} elm
     * @param {?} renderer
     * @param {?} zone
     */
    function ObserveEventDirective(elm, renderer, zone) {
        this.elm = elm;
        this.renderer = renderer;
        this.zone = zone;
        this.listeners = [];
    }
    Object.defineProperty(ObserveEventDirective.prototype, "events", {
        /**
         * @param {?} e
         * @return {?}
         */
        set: function (e) {
            var _this = this;
            this.unregisterAll();
            var /** @type {?} */ events = (typeof (e) === 'string')
                ? [e]
                : e;
            this.zone.runOutsideAngular(function () {
                _this.listeners = events.map(function (x) { return _this.addListener(x); });
            });
        },
        enumerable: true,
        configurable: true
    });
    /**
     * @return {?}
     */
    ObserveEventDirective.prototype.ngOnDestroy = function () {
        this.unregisterAll();
    };
    /**
     * @return {?}
     */
    ObserveEventDirective.prototype.unregisterAll = function () {
        this.listeners.forEach(function (x) { return x(); });
        this.listeners = [];
    };
    /**
     * @param {?} eventName
     * @return {?}
     */
    ObserveEventDirective.prototype.addListener = function (eventName) {
        var _this = this;
        var /** @type {?} */ colon = eventName.indexOf(':');
        var /** @type {?} */ handler = function ($event) { return _this.fireEvent($event); };
        if (colon === -1) {
            return this.renderer.listen(this.elm.nativeElement, eventName, handler);
        }
        else {
            var /** @type {?} */ scope = eventName.slice(0, colon);
            var /** @type {?} */ realEventName = eventName.slice(colon + 1);
            return this.renderer.listen(scope, realEventName, handler);
        }
    };
    /**
     * @param {?} $event
     * @return {?}
     */
    ObserveEventDirective.prototype.fireEvent = function ($event) {
        if (!this.observe)
            return;
        this.observe.next($event);
    };
    return ObserveEventDirective;
}());
ObserveEventDirective.decorators = [
    { type: Directive, args: [{
                selector: '[ev-observe]'
            },] },
];
/** @nocollapse */
ObserveEventDirective.ctorParameters = function () { return [
    { type: ElementRef, },
    { type: Renderer2, },
    { type: NgZone, },
]; };
ObserveEventDirective.propDecorators = {
    "observe": [{ type: Input, args: ['ev-observe',] },],
    "events": [{ type: Input, args: ['ev-events',] },],
};
/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
var ObserveEventDirectiveModule = (function () {
    function ObserveEventDirectiveModule() {
    }
    return ObserveEventDirectiveModule;
}());
ObserveEventDirectiveModule.decorators = [
    { type: NgModule, args: [{
                declarations: [
                    ObserveEventDirective
                ],
                exports: [
                    ObserveEventDirective
                ]
            },] },
];
/** @nocollapse */
ObserveEventDirectiveModule.ctorParameters = function () { return []; };
/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
/**
 * Quick-firing 'up' and 'down' events that work cross-browser for mouse and touch events
 *
 * Usage:
 * <button (down)="activate()" (up)="deactivate()"></button>
 *
 */
var TouchEventPlugin = (function (_super) {
    __extends(TouchEventPlugin, _super);
    /**
     * @param {?} doc
     */
    function TouchEventPlugin(doc) {
        return _super.call(this, doc) || this;
    }
    /**
     * @param {?} eventName
     * @return {?}
     */
    TouchEventPlugin.prototype.supports = function (eventName) {
        return eventName === 'down' || eventName === 'up';
    };
    /**
     * @param {?} element
     * @param {?} eventName
     * @param {?} handler
     * @return {?}
     */
    TouchEventPlugin.prototype.addEventListener = function (element, eventName, handler) {
        var _this = this;
        var /** @type {?} */ eventNames = (eventName === 'down')
            ? ['mousedown', 'touchstart', 'pointerdown']
            : ['mouseup', 'touchend', 'pointerup'];
        var /** @type {?} */ eventListeners = eventNames.map(function (x) { return _this.manager.addEventListener(element, x, function (e) {
            // prevent default so only one of the event listeners is fired
            e.preventDefault();
            handler(e);
        }); });
        return function () {
            eventListeners.forEach(function (x) { return x(); });
        };
    };
    return TouchEventPlugin;
}(MyEventManagerPlugin));
TouchEventPlugin.decorators = [
    { type: Injectable },
];
/** @nocollapse */
TouchEventPlugin.ctorParameters = function () { return [
    { type: undefined, decorators: [{ type: Inject, args: [DOCUMENT,] },] },
]; };
/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
var TouchEventModule = (function () {
    function TouchEventModule() {
    }
    return TouchEventModule;
}());
TouchEventModule.decorators = [
    { type: NgModule, args: [{
                providers: [
                    {
                        provide: EVENT_MANAGER_PLUGINS,
                        useClass: TouchEventPlugin,
                        multi: true
                    }
                ]
            },] },
];
/** @nocollapse */
TouchEventModule.ctorParameters = function () { return []; };
/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
/**
 * Automatically unregister an event listener after the event fired once
 *
 * Usage:
 * <button (once.click)="foo()">...</button>
 *
 */
var OnceEventPlugin = (function (_super) {
    __extends(OnceEventPlugin, _super);
    /**
     * @param {?} doc
     */
    function OnceEventPlugin(doc) {
        return _super.call(this, doc) || this;
    }
    /**
     * @param {?} eventName
     * @return {?}
     */
    OnceEventPlugin.prototype.supports = function (eventName) {
        return eventName.indexOf('once.') === 0;
    };
    /**
     * @param {?} element
     * @param {?} eventName
     * @param {?} handler
     * @return {?}
     */
    OnceEventPlugin.prototype.addEventListener = function (element, eventName, handler) {
        var /** @type {?} */ realEventName = eventName.slice(5);
        var /** @type {?} */ active = true;
        var /** @type {?} */ eventListener = this.manager.addEventListener(element, realEventName, function ($event) {
            eventListener();
            active = false;
            handler($event);
        });
        return function () {
            if (active)
                eventListener();
        };
    };
    return OnceEventPlugin;
}(MyEventManagerPlugin));
OnceEventPlugin.decorators = [
    { type: Injectable },
];
/** @nocollapse */
OnceEventPlugin.ctorParameters = function () { return [
    { type: undefined, decorators: [{ type: Inject, args: [DOCUMENT,] },] },
]; };
/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
var OnceEventModule = (function () {
    function OnceEventModule() {
    }
    return OnceEventModule;
}());
OnceEventModule.decorators = [
    { type: NgModule, args: [{
                providers: [
                    {
                        provide: EVENT_MANAGER_PLUGINS,
                        useClass: OnceEventPlugin,
                        multi: true
                    }
                ]
            },] },
];
/** @nocollapse */
OnceEventModule.ctorParameters = function () { return []; };
/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
/**
 * Conditionally apply event listeners to an element
 *
 * Usage:
 * <button [ev-condition]="isActive()"
 *      ev-events="click"
 *      (ev-fire)="handleClick($event)">...</button>
 *
 * Notes:
 * The [ev-events] property can be either a string or an array of strings to handle multiple events
 */
var ConditionEventDirective = (function () {
    /**
     * @param {?} elm
     * @param {?} renderer
     */
    function ConditionEventDirective(elm, renderer) {
        this.elm = elm;
        this.renderer = renderer;
        this.fire = new EventEmitter();
        this.listeners = [];
    }
    /**
     * @param {?} changes
     * @return {?}
     */
    ConditionEventDirective.prototype.ngOnChanges = function (changes) {
        var _this = this;
        this.unregisterAll();
        if (!this.condition || !this.events)
            return;
        var /** @type {?} */ events = (typeof (this.events) === 'string')
            ? [this.events]
            : this.events;
        this.listeners = events.map(function (x) { return _this.addListener(x); });
    };
    /**
     * @return {?}
     */
    ConditionEventDirective.prototype.ngOnDestroy = function () {
        this.unregisterAll();
    };
    /**
     * @return {?}
     */
    ConditionEventDirective.prototype.unregisterAll = function () {
        this.listeners.forEach(function (x) { return x(); });
        this.listeners = [];
    };
    /**
     * @param {?} eventName
     * @return {?}
     */
    ConditionEventDirective.prototype.addListener = function (eventName) {
        var _this = this;
        var /** @type {?} */ colon = eventName.indexOf(':');
        var /** @type {?} */ handler = function ($event) { return _this.fire.next($event); };
        if (colon === -1) {
            return this.renderer.listen(this.elm.nativeElement, eventName, handler);
        }
        else {
            var /** @type {?} */ scope = eventName.slice(0, colon);
            var /** @type {?} */ realEventName = eventName.slice(colon + 1);
            return this.renderer.listen(scope, realEventName, handler);
        }
    };
    return ConditionEventDirective;
}());
ConditionEventDirective.decorators = [
    { type: Directive, args: [{
                selector: '[ev-condition]'
            },] },
];
/** @nocollapse */
ConditionEventDirective.ctorParameters = function () { return [
    { type: ElementRef, },
    { type: Renderer2, },
]; };
ConditionEventDirective.propDecorators = {
    "condition": [{ type: Input, args: ['ev-condition',] },],
    "events": [{ type: Input, args: ['ev-events',] },],
    "fire": [{ type: Output, args: ['ev-fire',] },],
};
/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
var ConditionEventDirectiveModule = (function () {
    function ConditionEventDirectiveModule() {
    }
    return ConditionEventDirectiveModule;
}());
ConditionEventDirectiveModule.decorators = [
    { type: NgModule, args: [{
                declarations: [
                    ConditionEventDirective
                ],
                exports: [
                    ConditionEventDirective
                ]
            },] },
];
/** @nocollapse */
ConditionEventDirectiveModule.ctorParameters = function () { return []; };
/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
var SCROLL_EVENT_TIME = new InjectionToken('ScrollEventTime');
/**
 * Detects when an element is scrolled into or out of the viewport
 *
 * Usage:
 * <div (scroll-in)="activate()" (scroll-out)="deactivate()">...</div>
 *
 * The matching handler of the initial status is called upon attaching
 * to notify the element of its current status. $event is true for the
 * initial call, false otherwise
 *
 */
var ScrollEventPlugin = (function (_super) {
    __extends(ScrollEventPlugin, _super);
    /**
     * @param {?} doc
     * @param {?} time
     */
    function ScrollEventPlugin(doc, time) {
        var _this = _super.call(this, doc) || this;
        _this.listeners = [];
        _this.globalListener = undefined;
        _this.subject = new Subject$1();
        _this.subject
            .pipe(auditTime(time))
            .subscribe(function () {
            _this.listeners.forEach(function (x) { return x(); });
        });
        return _this;
    }
    /**
     * @param {?} eventName
     * @return {?}
     */
    ScrollEventPlugin.prototype.supports = function (eventName) {
        return eventName === 'scroll-in' || eventName === 'scroll-out';
    };
    /**
     * @param {?} element
     * @param {?} eventName
     * @param {?} handler
     * @return {?}
     */
    ScrollEventPlugin.prototype.addEventListener = function (element, eventName, handler) {
        var _this = this;
        var /** @type {?} */ isScrollIn = (eventName === 'scroll-in');
        var /** @type {?} */ status = undefined;
        setTimeout(function () {
            status = _this.getStatus(element);
            if ((isScrollIn && status) || (!isScrollIn && !status)) {
                handler(true);
            }
        }, 0);
        var /** @type {?} */ listener = function () {
            var /** @type {?} */ newStatus = _this.getStatus(element);
            if (status === newStatus)
                return;
            if ((isScrollIn && newStatus) || (!isScrollIn && !newStatus)) {
                _this.manager.getZone().run(function () { return handler(false); });
            }
            status = newStatus;
        };
        this.listeners.push(listener);
        this.updateGlobalListener();
        return function () {
            var /** @type {?} */ index = _this.listeners.indexOf(listener);
            if (index === -1)
                return;
            _this.listeners.splice(index, 1);
            _this.updateGlobalListener();
        };
    };
    /**
     * @return {?}
     */
    ScrollEventPlugin.prototype.updateGlobalListener = function () {
        var _this = this;
        if (this.listeners.length && this.globalListener === undefined) {
            this.manager.getZone().runOutsideAngular(function () {
                var /** @type {?} */ handler = function () {
                    _this.subject.next();
                };
                var /** @type {?} */ listeners = ['scroll', 'resize', 'orientationchange']
                    .map(function (x) { return _this.manager.addGlobalEventListener('window', x, handler); });
                _this.globalListener = function () {
                    listeners.forEach(function (x) { return x(); });
                };
            });
        }
        else if (!this.listeners.length && this.globalListener !== undefined) {
            this.globalListener();
            this.globalListener = undefined;
        }
    };
    /**
     * @param {?} element
     * @return {?}
     */
    ScrollEventPlugin.prototype.getStatus = function (element) {
        var /** @type {?} */ rect = element.getBoundingClientRect();
        return (rect.bottom >= 0 && rect.top <= window.innerHeight);
    };
    return ScrollEventPlugin;
}(MyEventManagerPlugin));
ScrollEventPlugin.decorators = [
    { type: Injectable },
];
/** @nocollapse */
ScrollEventPlugin.ctorParameters = function () { return [
    { type: undefined, decorators: [{ type: Inject, args: [DOCUMENT,] },] },
    { type: undefined, decorators: [{ type: Inject, args: [SCROLL_EVENT_TIME,] },] },
]; };
/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
var ScrollEventModule = (function () {
    function ScrollEventModule() {
    }
    return ScrollEventModule;
}());
ScrollEventModule.decorators = [
    { type: NgModule, args: [{
                providers: [
                    {
                        provide: EVENT_MANAGER_PLUGINS,
                        useClass: ScrollEventPlugin,
                        multi: true
                    },
                    {
                        provide: SCROLL_EVENT_TIME,
                        useValue: 200
                    }
                ]
            },] },
];
/** @nocollapse */
ScrollEventModule.ctorParameters = function () { return []; };
/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
var Ng2EventsModule$1 = (function () {
    function Ng2EventsModule$1() {
    }
    return Ng2EventsModule$1;
}());
Ng2EventsModule$1.decorators = [
    { type: NgModule, args: [{
                imports: [
                    OutsideEventModule,
                    TouchEventModule,
                    MultiEventModule,
                    UndetectedEventModule,
                    ObserveEventModule,
                    ObserveEventDirectiveModule,
                    OnceEventModule,
                    ConditionEventDirectiveModule,
                    ScrollEventModule
                ],
                exports: [
                    OutsideEventModule,
                    TouchEventModule,
                    MultiEventModule,
                    UndetectedEventModule,
                    ObserveEventModule,
                    ObserveEventDirectiveModule,
                    OnceEventModule,
                    ConditionEventDirectiveModule,
                    ScrollEventModule
                ]
            },] },
];
/** @nocollapse */
Ng2EventsModule$1.ctorParameters = function () { return []; };
/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
export { Ng2EventsModule$1 as Ng2EventsModule, OutsideEventModule, UndetectedEventModule, MultiEventModule, ObserveEventModule, ObserveEventDirectiveModule, TouchEventModule, OnceEventModule, ConditionEventDirectiveModule, ScrollEventModule, SCROLL_EVENT_TIME };
export default Ng2EventsModule$1;
